/*-
 * Copyright (c) 2001, Ramsey G. Brenner <rgbrenner@myrealbox.com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include "typingtutor.h"

extern char *home_dir;

void check(void)
{
    int      found_typingtutor_dir = FALSE;
    int      found_custom_dir = FALSE;
    int      found_log = FALSE;
    DIR     *dp;
    char    *log;
    char    *custom_dir;
    char    *typingtutor_dir;
    struct dirent   *dirp;

    /* Allocate space for file paths */
    if((custom_dir = malloc(strlen(home_dir) + sizeof("/.typingtutor/custom"))) == NULL) {
        fprintf(stderr, "Memory allocation error!\n");
        exit(EXIT_FAILURE);
    }
    strcpy(custom_dir, home_dir);
    strcat(custom_dir, "/.typingtutor/custom");
    if((typingtutor_dir = malloc(strlen(home_dir) + sizeof("/.typingtutor"))) == NULL) {
        fprintf(stderr, "Memory allocation error!\n");
        exit(EXIT_FAILURE);
    }
    strcpy(typingtutor_dir, home_dir);
    strcat(typingtutor_dir, "/.typingtutor");
    if((log = malloc(strlen(home_dir) + sizeof("/.typingtutor/log"))) == NULL) {
        fprintf(stderr, "Memory allocation error!\n");
        exit(EXIT_FAILURE);
    }
    strcpy(log, home_dir);
    strcat(log, "/.typingtutor/log");

    /* Look for files */
    if((dp = opendir(home_dir)) == NULL) {
        fprintf(stderr, "Cannot open `%s'!\n", home_dir);
        exit(EXIT_FAILURE);
    }
    while((dirp = readdir(dp)) != NULL && found_typingtutor_dir == FALSE) {
        if(!strcmp(dirp->d_name, ".typingtutor")) {
            found_typingtutor_dir = TRUE;
            break;
        }
    }
    closedir(dp);
    if(found_typingtutor_dir == TRUE) {
        if((dp = opendir(typingtutor_dir)) == NULL) {
            fprintf(stderr, "Cannot open `%s'!\n", typingtutor_dir);
            exit(EXIT_FAILURE);
        }
        while((dirp = readdir(dp)) != NULL) {
            if(!strcmp(dirp->d_name, "custom"))
                found_custom_dir = TRUE;
            else if(!strcmp(dirp->d_name, "log"))
                found_log = TRUE;
        }
        closedir(dp);
    }

    /* Create files if they don't exist */
    if(found_custom_dir == FALSE || found_log == FALSE || found_typingtutor_dir == FALSE) {
        if(found_typingtutor_dir == FALSE) {
            printf("Creating %s/.typingtutor/ ...\n", home_dir);
            mkdir(typingtutor_dir, S_IRUSR | S_IWUSR | S_IXUSR);
            printf("Creating %s/.typingtutor/ ...done\n", home_dir);
        }
        if(found_typingtutor_dir == FALSE || found_custom_dir == FALSE) {
            printf("Creating %s/.typingtutor/custom/ ...\n", home_dir);
            mkdir(custom_dir, S_IRUSR | S_IWUSR | S_IXUSR);
            printf("Creating %s/.typingtutor/custom/ ...done\n", home_dir);
        }
        if(found_typingtutor_dir == FALSE || found_log == FALSE) {
            printf("Creating %s/.typingtutor/log ...\n", home_dir);
            fclose(fopen(log, "w"));
            printf("Creating %s/.typingtutor/log ...done\n", home_dir);
        }
        sleep(2);
    }
    free(log);
    free(custom_dir);
    free(typingtutor_dir);
}
