/*-
 * Copyright (c) 2001, Ramsey G. Brenner <rgbrenner@myrealbox.com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include "typingtutor.h"

extern char *home_dir;
extern char *tt_dir;
extern char  lessons[15][15];

/* lesson fopen() */
FILE *lfopen(char *les, char *drill, char *mode)
{
    int      cnt;
    int      msg = FALSE;
    FILE    *fp;
    char    *file;

    if(*les == NULL) {
        if((file = malloc(strlen(drill) + 1)) == NULL) {
            fprintf(stderr, "Memory allocation error!\n");
            exit(EXIT_FAILURE);
        }
        strcpy(file, drill);
    } else if(!strcmp(les, "log")) {
        if((file = malloc(strlen(home_dir) + sizeof("/.typingtutor/log"))) == NULL) {
            fprintf(stderr, "Memory allocation error!\n");
            exit(EXIT_FAILURE);
        }
        strcpy(file, home_dir);
        strcat(file, "/.typingtutor/log");
    } else if(!strcmp(les, "custom")) {
        if((file = malloc(strlen(home_dir) + sizeof("/.typingtutor/custom/") + strlen(drill))) == NULL) {
            fprintf(stderr, "Memory allocation error!\n");
            exit(EXIT_FAILURE);
        }
        strcpy(file, home_dir);
        strcat(file, "/.typingtutor/custom/");
        strcat(file, drill);
        msg = TRUE;
    } else {
        if(tt_dir != NULL && tt_dir[strlen(tt_dir) - 2] != '/') {
            if(tt_dir[strlen(tt_dir) - 1] == '=')
                tt_dir[strlen(tt_dir) - 1] = NULL;
            if((file = malloc(strlen(tt_dir) + sizeof("//") + strlen(les) + strlen(drill))) == NULL) {
                fprintf(stderr, "Memory allocation error!\n");
                exit(EXIT_FAILURE);
            }
            strcpy(file, tt_dir);
            strcat(file, "/");
        } else if(tt_dir != NULL) {
            if(tt_dir[strlen(tt_dir) - 1] == '=')
                tt_dir[strlen(tt_dir) - 1] = NULL;
            if((file = malloc(strlen(tt_dir) + sizeof("/") + strlen(les) + strlen(drill))) == NULL) {
                fprintf(stderr, "Memory allocation error!\n");
                exit(EXIT_FAILURE);
            }
            strcpy(file, tt_dir);
        } else {
            if((file = malloc(sizeof("/usr/share/typingtutor//") + strlen(les) + strlen(drill))) == NULL) {
                fprintf(stderr, "Memory allocation error!\n");
                exit(EXIT_FAILURE);
            }
            strcpy(file, "/usr/share/typingtutor/");
        }
        strcat(file, les);
        strcat(file, "/");
        strcat(file, drill);
        msg = TRUE;
    }
    if(msg == TRUE) {
        for(cnt = 0; cnt < 16; cnt++)
            if(0 == strcmp(les, lessons[cnt]))
                break;
        if(cnt == 16) {
            fprintf(stderr, "Unknown lesson `%s'!\n", les);
            exit(EXIT_FAILURE);
        }
    }
    if((fp = fopen(file, mode)) == NULL) {
        if(msg == TRUE)
            fprintf(stderr, "Unknown drill `%s'\n", drill);
        else
            perror(0);
        exit(EXIT_FAILURE);
    }
    free(file);
    return(fp);
}
