/*-
 * Copyright (c) 2001, Ramsey G. Brenner <rgbrenner@myrealbox.com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include "typingtutor.h"

/* Global declarations should be made here */
char    *home_dir;
char    *tt_dir;
char     lessons[14][14] = {"home", "stretch", "reach", "topleft", "topright",
            "bottomleft", "capitalization", "puncuation", "lefthand",
            "righthand", "remaining", "numberpad", "skillbuilding", "custom"};

/* A simple typing tutor */
int main(int argc, char **argv)
{
    if(argc >= 2 && argc <= 6) {
        home_dir = getenv("HOME");
        tt_dir = getenv("TT_DIR");
        check();
        if(argc == 2) {
            if(!strcmp(argv[1], "history"))
                chart();
            else
                help();
        } else if(argc == 3) {
            if(!strcmp(argv[1], "new") && (!!strcmp(argv[2], "-q") && !!strcmp(argv[2], "--quiet")))
                new(argv[2]);
            else if(!strcmp(argv[1], "history") && (chk_lessn_grp(argv[2]) || !strcmp(argv[2], "all")))
                history(argv[2], NULL);
            else if(chk_lessn_grp(argv[1]))
                input(argc, argv[1], argv[2]);
            else
                help();
        } else if(argc == 4) {
            if((!strcmp(argv[1], "-q") || !strcmp(argv[1], "--quiet")) && chk_lessn_grp(argv[2]))
                input(argc, argv[2], argv[3]);
            else if(!strcmp(argv[1], "history") && chk_lessn_grp(argv[2]))
                history(argv[2], argv[3]);
            else
                help();
        } else if(argc == 5) {
            if(!strcmp(argv[1], "random")) {
                ran(argv[2], atoi(argv[3]), argv[4]);
                input(argc - 2, "custom", argv[4]);
            } else
                help();
        } else if(argc == 6) {
            if(!strcmp(argv[2], "random") &&
              (!strcmp(argv[1], "-q") || !strcmp(argv[1], "--quiet"))) {
                ran(argv[3], atoi(argv[4]), argv[5]);
                input(argc - 2, "custom", argv[5]);
            } else
                help();
        }
    } else
        help();
    return(EXIT_SUCCESS);
}

void help(void)
{
    fprintf(stderr, "%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s",
"Typing Tutor v0.6\n",
"Usage: typingtutor [-q] [history, new, random] [lesson group] [number] [lesson]\n\n",
"Options:\n",
"-q   Quiet mode\n\n",
"Lessons:\n",
"home <1-4, test>\n\
        Home keys\n\n",
"stretch <1-4, test>\n\
        Stretching out\n\n",
"reach <1-4, test>\n\
        Reaching down\n\n",
"topleft <1-4, test>\n\
        All the top left row\n\n",
"topright <1-4, test>\n\
        All the top right row\n\n",
"bottomleft <1-4, test>\n\
        All the bottom left row\n\n",
"capitalization <1-4, test>\n\
        Shift key/capitalization\n\n",
"puncuation <1-4, test>\n\
        Punctuation keys\n\n",
"lefthand <1-4, test>\n\
        Left-hand numbers and symbols\n\n",
"righthand <1-4, test>\n\
        Right-hand numbers and symbols\n\n",
"remaining <1-4, test>\n\
        All remaining keys\n\n",
"numberpad <1-4, test>\n\
        Number pad\n\n",
"skillbuilding <1-4, test>\n\
        Skill-building\n\n",
"custom <lesson>\n\
        Your Lessons\n\n\n",
"history [lesson group] [lesson]\n\
        Display history\n\n",
"new <lesson>\n\
        Create new <lesson> in <lesson group>\n\n",
"random <lesson group> <number> <lesson>\n\
        Create new <lesson> with random characters\n\n\n",
"SEE THE MAN PAGE FOR DESCRIPTIONS AND EXAMPLES\n");
}
