/*-
 * Copyright (c) 2001, Ramsey G. Brenner <rgbrenner@myrealbox.com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include "typingtutor.h"

void ran_numb(int number, char *file);

/* Produces random drills */
void ran(char *les, int char_numb, char *file)
{
    int      cnt;
    int      newline;
    int      devide;
    int      total;
    int      capital = FALSE;
    int      skillbuilding = FALSE;
    int      len_letters;
    int      rand_letter;
    int     *cnt_letters;
    char    *letters;
    FILE    *fp;

    if(!strcmp(les, "numberpad")) {
        ran_numb(char_numb, file);
        return;
    } else if(!strcmp(les, "home")) {
        if((letters = malloc(sizeof("asdf jkl;"))) == NULL) {
            fprintf(stderr, "Memory allocation error!\n");
            exit(EXIT_FAILURE);
        }
        strcpy(letters, "a;sl kdjf");
    } else if(!strcmp(les, "stretch")) {
        if((letters = malloc(sizeof("rutyhg rutyhg"))) == NULL) {
            fprintf(stderr, "Memory allocation error!\n");
            exit(EXIT_FAILURE);
        }
        strcpy(letters, "rutyhg rutyhg");
    } else if(!strcmp(les, "reach")) {
        if((letters = malloc(sizeof("vmnb vmnb"))) == NULL) {
            fprintf(stderr, "Memory allocation error!\n");
            exit(EXIT_FAILURE);
        }
        strcpy(letters, "vmnb vmnb");
    } else if(!strcmp(les, "topleft")) {
        if((letters = malloc(sizeof("qe wqe wqe w"))) == NULL) {
            fprintf(stderr, "Memory allocation error!\n");
            exit(EXIT_FAILURE);
        }
        strcpy(letters, "qe wqe wqe w");
    } else if(!strcmp(les, "topright")) {
        if((letters = malloc(sizeof("ip oip oip o"))) == NULL) {
            fprintf(stderr, "Memory allocation error!\n");
            exit(EXIT_FAILURE);
        }
        strcpy(letters, "ip oip oip o");
    } else if(!strcmp(les, "bottomleft")) {
        if((letters = malloc(sizeof("zc xzc xzc x"))) == NULL) {
            fprintf(stderr, "Memory allocation error!\n");
            exit(EXIT_FAILURE);
        }
        strcpy(letters, "zc xzc xzc x");
    } else if(!strcmp(les, "lefthand")) {
        if((letters = malloc(sizeof("1!2@3# 4$5%6^"))) == NULL) {
            fprintf(stderr, "Memory allocation error!\n");
            exit(EXIT_FAILURE);
        }
        strcpy(letters, "1!2@3# 4$5%6^");
    } else if(!strcmp(les, "righthand")) {
        if((letters = malloc(sizeof("7&8* 9(0)"))) == NULL) {
            fprintf(stderr, "Memory allocation error!\n");
            exit(EXIT_FAILURE);
        }
        strcpy(letters, "7&8* 9(0)");
    } else if(!strcmp(les, "capitalization")) {
        capital = TRUE;
        if((letters = malloc(sizeof("abc defghijk lmnopqrst uvwxyz"))) == NULL) {
            fprintf(stderr, "Memory allocation error!\n");
            exit(EXIT_FAILURE);
        }
        strcpy(letters, "abc defghijk lmnopqrst uvwxyz");
    } else if(!strcmp(les, "remaining")) {
        if((letters = malloc(sizeof("`-=[]\\ :_+ {|< >}//'\" "))) == NULL) {
            fprintf(stderr, "Memory allocation error!\n");
            exit(EXIT_FAILURE);
        }
        strcpy(letters, "`-=[]\\ :_+ {|< >}/'\"");
    } else if(!strcmp(les, "puncuation")) {
        if((letters = malloc(sizeof("abc defghijk lmnopqrst uvwxyz.?:! .?:! .?:!"))) == NULL) {
            fprintf(stderr, "Memory allocation error!\n");
            exit(EXIT_FAILURE);
        }
        strcpy(letters, "abc defghijk lmnopqrst uvwxyz .?:! .?:! .?:!");
    } else if(!strcmp(les, "skillbuilding")) {
        skillbuilding = TRUE;
        if((letters = malloc(sizeof("abc defghijk lmnopqrst uvwxyz .?:! `-=[]\\ :_+ {|< >}//'\" 1!2@3# 4$5%6^ 7&8* 9(0)"))) == NULL) {
            fprintf(stderr, "Memory allocation error!\n");
            exit(EXIT_FAILURE);
        }
        strcpy(letters, "abc defghijk lmnopqrst uvwxyz .?:! `-=[]\\ :_+ {|< >}//'\" 1!2@3# 4$5%6^ 7&8* 9(0)");
    } else {
        fprintf(stderr, "Invalid lesson `%s'!\n", les);
        exit(EXIT_FAILURE);
    }
    srandom(time(0) * getpid());
    fp = lfopen("custom", file, "w");
        len_letters = strlen(letters);
        devide = len_letters + 1;
    if((cnt_letters = malloc(sizeof(*cnt_letters) * len_letters)) == NULL) {
        fprintf(stderr, "Memory allocation error!\n");
        exit(EXIT_FAILURE);
    }
    for(cnt = 0; cnt < len_letters; cnt++)
        cnt_letters[cnt] = 0;
    for(newline = 0, total = 0; total < char_numb; ) {
        rand_letter = (int)random();
        rand_letter %= devide;
        rand_letter -= 1;
        if(rand_letter < 0)
            continue;
        for(cnt = 0; cnt < len_letters; cnt++)
            if(cnt_letters[rand_letter] - cnt_letters[cnt] >= 1)
                break;
        if(cnt != len_letters)
            continue;
        if(letters[rand_letter] == ' ' && (newline >= 72 || newline <= 2 || char_numb - total <= 2))
            continue;
        if(capital == TRUE && total % 3)
            fprintf(fp, "%c", toupper(letters[rand_letter]));
        else if(skillbuilding == TRUE && !(total % 5))
            fprintf(fp, "%c", toupper(letters[rand_letter]));
        else
            fprintf(fp, "%c", letters[rand_letter]);
        cnt_letters[rand_letter]++;
        total++;
        newline++;
        if(newline >= 74) {
            fprintf(fp, "\n");
            newline = 0;
        }
    }
    fprintf(fp, "\n\n");
    free(letters);
    free(cnt_letters);
    fclose(fp);
}

void ran_numb(int number, char *file)
{
    int      cnt;
    int      numb;
    int      total;
    int      newline;
    int      rand_newline;
    int     *cnt_numb;
    FILE    *fp;

    srandom(time(0) * getpid());
    fp = lfopen("custom", file, "w");
    if((cnt_numb = malloc(sizeof(*cnt_numb) * 10)) == NULL) {
        fprintf(stderr, "Memory allocation error!\n");
        exit(EXIT_FAILURE);
    }
    for(cnt = 0; cnt < 10; cnt++)
        cnt_numb[cnt] = 0;
    rand_newline = (int)random();
    while(rand_newline >= 15)
        rand_newline /= 10;
    if(rand_newline <= 5)
        rand_newline += 5;
    for(total = 0, newline = 0; total < number; ) {
        numb = (int)random();
        numb %= 10;
        for(cnt = 0; cnt < 10; cnt++)
            if((cnt_numb[numb] - cnt_numb[cnt]) >= 1)
                break;
        if(cnt != 10)
            continue;
        fprintf(fp, "%d", numb);
        total++;
        newline++;
        cnt_numb[numb]++;
        if(newline == rand_newline) {
            newline = 0;
            fprintf(fp, "\n");
            rand_newline = (int)random();
            while(rand_newline >= 15)
                rand_newline /= 10;
            if(rand_newline <= 5)
                rand_newline += 5;
        }
    }
    fprintf(fp, "\n\n");
    free(cnt_numb);
    fclose(fp);
}
